/**
 * An expression language of ScopePathStr with a binary comma (to indicate alternatives) operator.
 * Examples: `foo.bar boo.baz,quick quack`
*/
declare type ScopePattern = string;
/**
 * A TextMate theme.
 */
interface IRawTheme {
    readonly name?: string;
    readonly settings: IRawThemeSetting[];
}
/**
 * A single theme setting.
 */
interface IRawThemeSetting {
    readonly name?: string;
    readonly scope?: ScopePattern | ScopePattern[];
    readonly settings: {
        readonly fontStyle?: string;
        readonly foreground?: string;
        readonly background?: string;
    };
}

/**
 * A grammar
 */
interface IGrammar {
    /**
     * Tokenize `lineText` using previous line state `prevState`.
     */
    tokenizeLine(lineText: string, prevState: StateStack | null, timeLimit?: number): ITokenizeLineResult;
    /**
     * Tokenize `lineText` using previous line state `prevState`.
     * The result contains the tokens in binary format, resolved with the following information:
     *  - language
     *  - token type (regex, string, comment, other)
     *  - font style
     *  - foreground color
     *  - background color
     * e.g. for getting the languageId: `(metadata & MetadataConsts.LANGUAGEID_MASK) >>> MetadataConsts.LANGUAGEID_OFFSET`
     */
    tokenizeLine2(lineText: string, prevState: StateStack | null, timeLimit?: number): ITokenizeLineResult2;
}
interface ITokenizeLineResult {
    readonly tokens: IToken[];
    /**
     * The `prevState` to be passed on to the next line tokenization.
     */
    readonly ruleStack: StateStack;
    /**
     * Did tokenization stop early due to reaching the time limit.
     */
    readonly stoppedEarly: boolean;
}
interface ITokenizeLineResult2 {
    /**
     * The tokens in binary format. Each token occupies two array indices. For token i:
     *  - at offset 2*i => startIndex
     *  - at offset 2*i + 1 => metadata
     *
     */
    readonly tokens: Uint32Array;
    /**
     * The `prevState` to be passed on to the next line tokenization.
     */
    readonly ruleStack: StateStack;
    /**
     * Did tokenization stop early due to reaching the time limit.
     */
    readonly stoppedEarly: boolean;
}
interface IToken {
    startIndex: number;
    readonly endIndex: number;
    readonly scopes: string[];
}
/**
 * **IMPORTANT** - Immutable!
 */
interface StateStack {
    _stackElementBrand: void;
    readonly depth: number;
    clone(): StateStack;
    equals(other: StateStack): boolean;
}

declare const languages: {
    abap: () => Promise<LanguageRegistration>;
    'actionscript-3': () => Promise<LanguageRegistration>;
    ada: () => Promise<LanguageRegistration>;
    apache: () => Promise<LanguageRegistration>;
    apex: () => Promise<LanguageRegistration>;
    apl: () => Promise<LanguageRegistration>;
    applescript: () => Promise<LanguageRegistration>;
    ara: () => Promise<LanguageRegistration>;
    asm: () => Promise<LanguageRegistration>;
    astro: () => Promise<LanguageRegistration>;
    awk: () => Promise<LanguageRegistration>;
    ballerina: () => Promise<LanguageRegistration>;
    bat: () => Promise<LanguageRegistration>;
    beancount: () => Promise<LanguageRegistration>;
    berry: () => Promise<LanguageRegistration>;
    bibtex: () => Promise<LanguageRegistration>;
    bicep: () => Promise<LanguageRegistration>;
    blade: () => Promise<LanguageRegistration>;
    c: () => Promise<LanguageRegistration>;
    cadence: () => Promise<LanguageRegistration>;
    clarity: () => Promise<LanguageRegistration>;
    clojure: () => Promise<LanguageRegistration>;
    cmake: () => Promise<LanguageRegistration>;
    cobol: () => Promise<LanguageRegistration>;
    codeql: () => Promise<LanguageRegistration>;
    coffee: () => Promise<LanguageRegistration>;
    cpp: () => Promise<LanguageRegistration>;
    crystal: () => Promise<LanguageRegistration>;
    csharp: () => Promise<LanguageRegistration>;
    css: () => Promise<LanguageRegistration>;
    cue: () => Promise<LanguageRegistration>;
    cypher: () => Promise<LanguageRegistration>;
    d: () => Promise<LanguageRegistration>;
    dart: () => Promise<LanguageRegistration>;
    dax: () => Promise<LanguageRegistration>;
    diff: () => Promise<LanguageRegistration>;
    docker: () => Promise<LanguageRegistration>;
    'dream-maker': () => Promise<LanguageRegistration>;
    elixir: () => Promise<LanguageRegistration>;
    elm: () => Promise<LanguageRegistration>;
    erb: () => Promise<LanguageRegistration>;
    erlang: () => Promise<LanguageRegistration>;
    fish: () => Promise<LanguageRegistration>;
    fsharp: () => Promise<LanguageRegistration>;
    gdresource: () => Promise<LanguageRegistration>;
    gdscript: () => Promise<LanguageRegistration>;
    gdshader: () => Promise<LanguageRegistration>;
    gherkin: () => Promise<LanguageRegistration>;
    'git-commit': () => Promise<LanguageRegistration>;
    'git-rebase': () => Promise<LanguageRegistration>;
    'glimmer-js': () => Promise<LanguageRegistration>;
    'glimmer-ts': () => Promise<LanguageRegistration>;
    glsl: () => Promise<LanguageRegistration>;
    gnuplot: () => Promise<LanguageRegistration>;
    go: () => Promise<LanguageRegistration>;
    graphql: () => Promise<LanguageRegistration>;
    groovy: () => Promise<LanguageRegistration>;
    hack: () => Promise<LanguageRegistration>;
    haml: () => Promise<LanguageRegistration>;
    handlebars: () => Promise<LanguageRegistration>;
    haskell: () => Promise<LanguageRegistration>;
    hcl: () => Promise<LanguageRegistration>;
    hjson: () => Promise<LanguageRegistration>;
    hlsl: () => Promise<LanguageRegistration>;
    html: () => Promise<LanguageRegistration>;
    http: () => Promise<LanguageRegistration>;
    imba: () => Promise<LanguageRegistration>;
    ini: () => Promise<LanguageRegistration>;
    java: () => Promise<LanguageRegistration>;
    javascript: () => Promise<LanguageRegistration>;
    'jinja-html': () => Promise<LanguageRegistration>;
    jison: () => Promise<LanguageRegistration>;
    json: () => Promise<LanguageRegistration>;
    json5: () => Promise<LanguageRegistration>;
    jsonc: () => Promise<LanguageRegistration>;
    jsonl: () => Promise<LanguageRegistration>;
    jsonnet: () => Promise<LanguageRegistration>;
    jssm: () => Promise<LanguageRegistration>;
    jsx: () => Promise<LanguageRegistration>;
    julia: () => Promise<LanguageRegistration>;
    kotlin: () => Promise<LanguageRegistration>;
    kusto: () => Promise<LanguageRegistration>;
    latex: () => Promise<LanguageRegistration>;
    less: () => Promise<LanguageRegistration>;
    liquid: () => Promise<LanguageRegistration>;
    lisp: () => Promise<LanguageRegistration>;
    logo: () => Promise<LanguageRegistration>;
    lua: () => Promise<LanguageRegistration>;
    make: () => Promise<LanguageRegistration>;
    markdown: () => Promise<LanguageRegistration>;
    marko: () => Promise<LanguageRegistration>;
    matlab: () => Promise<LanguageRegistration>;
    mdx: () => Promise<LanguageRegistration>;
    mermaid: () => Promise<LanguageRegistration>;
    narrat: () => Promise<LanguageRegistration>;
    nextflow: () => Promise<LanguageRegistration>;
    nginx: () => Promise<LanguageRegistration>;
    nim: () => Promise<LanguageRegistration>;
    nix: () => Promise<LanguageRegistration>;
    'objective-c': () => Promise<LanguageRegistration>;
    'objective-cpp': () => Promise<LanguageRegistration>;
    ocaml: () => Promise<LanguageRegistration>;
    pascal: () => Promise<LanguageRegistration>;
    perl: () => Promise<LanguageRegistration>;
    php: () => Promise<LanguageRegistration>;
    plsql: () => Promise<LanguageRegistration>;
    postcss: () => Promise<LanguageRegistration>;
    powerquery: () => Promise<LanguageRegistration>;
    powershell: () => Promise<LanguageRegistration>;
    prisma: () => Promise<LanguageRegistration>;
    prolog: () => Promise<LanguageRegistration>;
    proto: () => Promise<LanguageRegistration>;
    pug: () => Promise<LanguageRegistration>;
    puppet: () => Promise<LanguageRegistration>;
    purescript: () => Promise<LanguageRegistration>;
    python: () => Promise<LanguageRegistration>;
    r: () => Promise<LanguageRegistration>;
    raku: () => Promise<LanguageRegistration>;
    razor: () => Promise<LanguageRegistration>;
    reg: () => Promise<LanguageRegistration>;
    rel: () => Promise<LanguageRegistration>;
    riscv: () => Promise<LanguageRegistration>;
    rst: () => Promise<LanguageRegistration>;
    ruby: () => Promise<LanguageRegistration>;
    rust: () => Promise<LanguageRegistration>;
    sas: () => Promise<LanguageRegistration>;
    sass: () => Promise<LanguageRegistration>;
    scala: () => Promise<LanguageRegistration>;
    scheme: () => Promise<LanguageRegistration>;
    scss: () => Promise<LanguageRegistration>;
    shaderlab: () => Promise<LanguageRegistration>;
    shellscript: () => Promise<LanguageRegistration>;
    smalltalk: () => Promise<LanguageRegistration>;
    solidity: () => Promise<LanguageRegistration>;
    sparql: () => Promise<LanguageRegistration>;
    sql: () => Promise<LanguageRegistration>;
    'ssh-config': () => Promise<LanguageRegistration>;
    stata: () => Promise<LanguageRegistration>;
    stylus: () => Promise<LanguageRegistration>;
    svelte: () => Promise<LanguageRegistration>;
    swift: () => Promise<LanguageRegistration>;
    'system-verilog': () => Promise<LanguageRegistration>;
    tasl: () => Promise<LanguageRegistration>;
    tcl: () => Promise<LanguageRegistration>;
    tex: () => Promise<LanguageRegistration>;
    toml: () => Promise<LanguageRegistration>;
    tsx: () => Promise<LanguageRegistration>;
    turtle: () => Promise<LanguageRegistration>;
    twig: () => Promise<LanguageRegistration>;
    typescript: () => Promise<LanguageRegistration>;
    v: () => Promise<LanguageRegistration>;
    vb: () => Promise<LanguageRegistration>;
    verilog: () => Promise<LanguageRegistration>;
    vhdl: () => Promise<LanguageRegistration>;
    viml: () => Promise<LanguageRegistration>;
    'vue-html': () => Promise<LanguageRegistration>;
    vue: () => Promise<LanguageRegistration>;
    vyper: () => Promise<LanguageRegistration>;
    wasm: () => Promise<LanguageRegistration>;
    wenyan: () => Promise<LanguageRegistration>;
    wgsl: () => Promise<LanguageRegistration>;
    wolfram: () => Promise<LanguageRegistration>;
    xml: () => Promise<LanguageRegistration>;
    xsl: () => Promise<LanguageRegistration>;
    yaml: () => Promise<LanguageRegistration>;
    zenscript: () => Promise<LanguageRegistration>;
};

declare const themes: {
    'css-variables': () => Promise<ThemeRegisterationRaw>;
    'dark-plus': () => Promise<ThemeRegisterationRaw>;
    'dracula-soft': () => Promise<ThemeRegisterationRaw>;
    dracula: () => Promise<ThemeRegisterationRaw>;
    'github-dark-dimmed': () => Promise<ThemeRegisterationRaw>;
    'github-dark': () => Promise<ThemeRegisterationRaw>;
    'github-light': () => Promise<ThemeRegisterationRaw>;
    hc_light: () => Promise<ThemeRegisterationRaw>;
    'light-plus': () => Promise<ThemeRegisterationRaw>;
    'material-theme-darker': () => Promise<ThemeRegisterationRaw>;
    'material-theme-lighter': () => Promise<ThemeRegisterationRaw>;
    'material-theme-ocean': () => Promise<ThemeRegisterationRaw>;
    'material-theme-palenight': () => Promise<ThemeRegisterationRaw>;
    'material-theme': () => Promise<ThemeRegisterationRaw>;
    'min-dark': () => Promise<ThemeRegisterationRaw>;
    'min-light': () => Promise<ThemeRegisterationRaw>;
    monokai: () => Promise<ThemeRegisterationRaw>;
    nord: () => Promise<ThemeRegisterationRaw>;
    'one-dark-pro': () => Promise<ThemeRegisterationRaw>;
    poimandres: () => Promise<ThemeRegisterationRaw>;
    'rose-pine-dawn': () => Promise<ThemeRegisterationRaw>;
    'rose-pine-moon': () => Promise<ThemeRegisterationRaw>;
    'rose-pine': () => Promise<ThemeRegisterationRaw>;
    'slack-dark': () => Promise<ThemeRegisterationRaw>;
    'slack-ochin': () => Promise<ThemeRegisterationRaw>;
    'solarized-dark': () => Promise<ThemeRegisterationRaw>;
    'solarized-light': () => Promise<ThemeRegisterationRaw>;
    'vitesse-dark': () => Promise<ThemeRegisterationRaw>;
    'vitesse-light': () => Promise<ThemeRegisterationRaw>;
};

declare enum FontStyle {
    NotSet = -1,
    None = 0,
    Italic = 1,
    Bold = 2,
    Underline = 4
}

interface IThemedTokenScopeExplanation {
    scopeName: string;
    themeMatches: any[];
}
interface IThemedTokenExplanation {
    content: string;
    scopes: IThemedTokenScopeExplanation[];
}
/**
 * A single token with color, and optionally with explanation.
 *
 * For example:
 *
 * {
 *   "content": "shiki",
 *   "color": "#D8DEE9",
 *   "explanation": [
 *     {
 *       "content": "shiki",
 *       "scopes": [
 *         {
 *           "scopeName": "source.js",
 *           "themeMatches": []
 *         },
 *         {
 *           "scopeName": "meta.objectliteral.js",
 *           "themeMatches": []
 *         },
 *         {
 *           "scopeName": "meta.object.member.js",
 *           "themeMatches": []
 *         },
 *         {
 *           "scopeName": "meta.array.literal.js",
 *           "themeMatches": []
 *         },
 *         {
 *           "scopeName": "variable.other.object.js",
 *           "themeMatches": [
 *             {
 *               "name": "Variable",
 *               "scope": "variable.other",
 *               "settings": {
 *                 "foreground": "#D8DEE9"
 *               }
 *             },
 *             {
 *               "name": "[JavaScript] Variable Other Object",
 *               "scope": "source.js variable.other.object",
 *               "settings": {
 *                 "foreground": "#D8DEE9"
 *               }
 *             }
 *           ]
 *         }
 *       ]
 *     }
 *   ]
 * }
 *
 */
interface IThemedToken {
    /**
     * The content of the token
     */
    content: string;
    /**
     * 6 or 8 digit hex code representation of the token's color
     */
    color?: string;
    /**
     * Font style of token. Can be None/Italic/Bold/Underline
     */
    fontStyle?: FontStyle;
    /**
     * Explanation of
     *
     * - token text's matching scopes
     * - reason that token text is given a color (one matching scope matches a rule (scope -> color) in the theme)
     */
    explanation?: IThemedTokenExplanation[];
}

type BuiltinLanguages = keyof typeof languages;
type BuiltinThemes = keyof typeof themes;
type Awaitable<T> = T | Promise<T>;
type MaybeGetter<T> = T | (() => Awaitable<T>);
type ThemeInput = MaybeGetter<ThemeRegisteration | ThemeRegisterationRaw>;
type LanguageInput = MaybeGetter<LanguageRegistration>;
interface LanguageRegistration {
    id: string;
    scopeName: string;
    displayName?: string;
    aliases?: string[];
    /**
     * A list of languages the current language embeds.
     * If manually specifying languages to load, make sure to load the embedded
     * languages for each parent language.
     */
    embeddedLangs?: string[];
    balancedBracketSelectors?: string[];
    unbalancedBracketSelectors?: string[];
    grammar?: IGrammar;
}
interface CodeToHtmlOptions {
    lang?: string;
    theme?: string;
    lineOptions?: LineOption[];
}
interface LineOption {
    /**
     * 1-based line number.
     */
    line: number;
    classes?: string[];
}
interface ThemeRegisterationRaw extends IRawTheme {
}
interface ThemeRegisteration extends ThemeRegisterationRaw {
    /**
     * @description theme name
     */
    name: string;
    /**
     * @description light/dark theme
     */
    type: 'light' | 'dark' | 'css';
    /**
     * @description tokenColors of the theme file
     */
    settings: any[];
    /**
     * @description text default foreground color
     */
    fg: string;
    /**
     * @description text default background color
     */
    bg: string;
    /**
     * @description relative path of included theme
     */
    include?: string;
    /**
     *
     * @description color map of the theme file
     */
    colors?: Record<string, string>;
}
interface HtmlRendererOptions {
    langId?: string;
    fg?: string;
    bg?: string;
    lineOptions?: LineOption[];
    elements?: ElementsOptions;
    themeName?: string;
}
interface ElementsOptions {
    pre?: (props: PreElementProps) => string;
    code?: (props: CodeElementProps) => string;
    line?: (props: LineElementProps) => string;
    token?: (props: TokenElementProps) => string;
}
interface ElementProps {
    children: string;
    [key: string]: unknown;
}
interface PreElementProps extends ElementProps {
    className: string;
    style: string;
}
interface CodeElementProps extends ElementProps {
}
interface LineElementProps extends ElementProps {
    className: string;
    lines: IThemedToken[][];
    line: IThemedToken[];
    index: number;
}
interface TokenElementProps extends ElementProps {
    style: string;
    tokens: IThemedToken[];
    token: IThemedToken;
    index: number;
}

export { Awaitable, BuiltinLanguages, BuiltinThemes, CodeToHtmlOptions, ElementsOptions, HtmlRendererOptions, LanguageInput, LanguageRegistration, LineOption, MaybeGetter, ThemeInput, ThemeRegisteration, ThemeRegisterationRaw };
